﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;


namespace TinyMCE
{
    public partial class TinyMCEditor : Form
    {
        public TreeNode SelectedNode = null;


        // for more details please see w3c.org
        // now here is the complete list ;)
        string[,] charmap = new string[,]{
//Spacing
    {"&nbsp;",    "&#160;",  " ", "no-break space"},
	{"&quot;",    "&#34;",   "\"", "quotation mark"},
    //{"&amp;",     "&#38;",   "&", "ampersand"}
// finance
	{"&cent;",    "&#162;",  "¢", "cent sign"},
	{"&euro;",    "&#8364;", "€", "euro sign"},
	{"&pound;",   "&#163;",  "£", "pound sign"},
	{"&yen;",     "&#165;",  "¥", "yen sign"},
// signs
	{"&copy;",    "&#169;",  "©", "copyright sign"},
	{"&reg;",     "&#174;",  "®", "registered sign"},
	{"&trade;",   "&#8482;", "™", "trade mark sign"},
	{"&permil;",  "&#8240;", "‰", "per mille sign"},
	{"&micro;",   "&#181;",  "µ", "micro sign"},
	{"&middot;",  "&#183;",  "·", "middle dot"},
	{"&bull;",    "&#8226;", "•", "bullet"},
	{"&hellip;",  "&#8230;", "…", "three dot leader"},
	{"&prime;",   "&#8242;", "′", "minutes / feet"},
	{"&Prime;",   "&#8243;", "″", "seconds / inches"},
	{"&sect;",    "&#167;",  "§", "section sign"},
	{"&para;",    "&#182;",  "¶", "paragraph sign"},
	{"&szlig;",   "&#223;",  "ß", "sharp s / ess-zed"},
// quotations
	{"&lsaquo;",  "&#8249;", "‹", "single left-pointing angle quotation mark"},
	{"&rsaquo;",  "&#8250;", "›", "single right-pointing angle quotation mark"},
	{"&laquo;",   "&#171;",  "«", "left pointing guillemet"},
	{"&raquo;",   "&#187;",  "»", "right pointing guillemet"},
	{"&lsquo;",   "&#8216;", "‘", "left single quotation mark"},
	{"&rsquo;",   "&#8217;", "’", "right single quotation mark"},
	{"&ldquo;",   "&#8220;", "“", "left double quotation mark"},
	{"&rdquo;",   "&#8221;", "”", "right double quotation mark"},
	{"&sbquo;",   "&#8218;", "‚", "single low-9 quotation mark"},
	{"&bdquo;",   "&#8222;", "„", "double low-9 quotation mark"},
	{"&lt;",      "&#60;",   "<", "less-than sign"},
	{"&gt;",      "&#62;",   ">", "greater-than sign"},
	{"&le;",      "&#8804;", "≤", "less-than or equal to"},
	{"&ge;",      "&#8805;", "≥", "greater-than or equal to"},
	{"&ndash;",   "&#8211;", "–", "en dash"},
	{"&mdash;",   "&#8212;", "—", "em dash"},
	{"&macr;",    "&#175;",  "¯", "macron"},
	{"&oline;",   "&#8254;", "‾", "overline"},
	{"&curren;",  "&#164;",  "¤", "currency sign"},
	{"&brvbar;",  "&#166;",  "¦", "broken bar"},
	{"&uml;",     "&#168;",  "¨", "diaeresis"},
	{"&iexcl;",   "&#161;",  "¡", "inverted exclamation mark"},
	{"&iquest;",  "&#191;",  "¿", "turned question mark"},
	{"&circ;",    "&#710;",  "ˆ", "circumflex accent"},
	{"&tilde;",   "&#732;",  "˜", "small tilde"},
	{"&deg;",     "&#176;",  "°", "degree sign"},
	{"&minus;",   "&#8722;", "−", "minus sign"},
	{"&plusmn;",  "&#177;",  "±", "plus-minus sign"},
	{"&divide;",  "&#247;",  "÷", "division sign"},
	{"&frasl;",   "&#8260;", "⁄", "fraction slash"},
	{"&times;",   "&#215;",  "×", "multiplication sign"},
	{"&sup1;",    "&#185;",  "¹", "superscript one"},
	{"&sup2;",    "&#178;",  "²", "superscript two"},
	{"&sup3;",    "&#179;",  "³", "superscript three"},
	{"&frac14;",  "&#188;",  "¼", "fraction one quarter"},
	{"&frac12;",  "&#189;",  "½", "fraction one half"},
	{"&frac34;",  "&#190;",  "¾", "fraction three quarters"},
// math / logical
	{"&fnof;",    "&#402;",  "ƒ", "function / florin"},
	{"&int;",     "&#8747;", "∫", "integral"},
	{"&sum;",     "&#8721;", "∑", "n-ary sumation"},
	{"&infin;",   "&#8734;", "∞", "infinity"},
	{"&radic;",   "&#8730;", "√", "square root"},
	{"&sim;",     "&#8764;", "∼","similar to"},
	{"&cong;",    "&#8773;", "≅","approximately equal to"},
	{"&asymp;",   "&#8776;", "≈", "almost equal to"},
	{"&ne;",      "&#8800;", "≠", "not equal to"},
	{"&equiv;",   "&#8801;", "≡", "identical to"},
	{"&isin;",    "&#8712;", "∈","element of"},
	{"&notin;",   "&#8713;", "∉","not an element of"},
	{"&ni;",      "&#8715;", "∋","contains as member"},
	{"&prod;",    "&#8719;", "∏", "n-ary product"},
	{"&and;",     "&#8743;", "∧","logical and"},
	{"&or;",      "&#8744;", "∨","logical or"},
	{"&not;",     "&#172;",  "¬", "not sign"},
	{"&cap;",     "&#8745;", "∩", "intersection"},
	{"&cup;",     "&#8746;", "∪","union"},
	{"&part;",    "&#8706;", "∂", "partial differential"},
	{"&forall;",  "&#8704;", "∀","for all"},
	{"&exist;",   "&#8707;", "∃","there exists"},
	{"&empty;",   "&#8709;", "∅","diameter"},
	{"&nabla;",   "&#8711;", "∇","backward difference"},
	{"&lowast;",  "&#8727;", "∗","asterisk operator"},
	{"&prop;",    "&#8733;", "∝","proportional to"},
	{"&ang;",     "&#8736;", "∠","angle"},
// undefined
	{"&acute;",   "&#180;",  "´", "acute accent"},
	{"&cedil;",   "&#184;",  "¸", "cedilla"},
	{"&ordf;",    "&#170;",  "ª", "feminine ordinal indicator"},
	{"&ordm;",    "&#186;",  "º", "masculine ordinal indicator"},
	{"&dagger;",  "&#8224;", "†", "dagger"},
	{"&Dagger;",  "&#8225;", "‡", "double dagger"},
// alphabetical special chars
	{"&Agrave;",  "&#192;",  "À", "A - grave"},
	{"&Aacute;",  "&#193;",  "Á", "A - acute"},
	{"&Acirc;",   "&#194;",  "Â", "A - circumflex"},
	{"&Atilde;",  "&#195;",  "Ã", "A - tilde"},
	{"&Auml;",    "&#196;",  "Ä", "A - diaeresis"},
	{"&Aring;",   "&#197;",  "Å", "A - ring above"},
	{"&AElig;",   "&#198;",  "Æ", "ligature AE"},
	{"&Ccedil;",  "&#199;",  "Ç", "C - cedilla"},
	{"&Egrave;",  "&#200;",  "È", "E - grave"},
	{"&Eacute;",  "&#201;",  "É", "E - acute"},
	{"&Ecirc;",   "&#202;",  "Ê", "E - circumflex"},
	{"&Euml;",    "&#203;",  "Ë", "E - diaeresis"},
	{"&Igrave;",  "&#204;",  "Ì", "I - grave"},
	{"&Iacute;",  "&#205;",  "Í", "I - acute"},
	{"&Icirc;",   "&#206;",  "Î", "I - circumflex"},
	{"&Iuml;",    "&#207;",  "Ï", "I - diaeresis"},
	{"&ETH;",     "&#208;",  "Ð", "ETH"},
	{"&Ntilde;",  "&#209;",  "Ñ", "N - tilde"},
	{"&Ograve;",  "&#210;",  "Ò", "O - grave"},
	{"&Oacute;",  "&#211;",  "Ó", "O - acute"},
	{"&Ocirc;",   "&#212;",  "Ô", "O - circumflex"},
	{"&Otilde;",  "&#213;",  "Õ", "O - tilde"},
	{"&Ouml;",    "&#214;",  "Ö", "O - diaeresis"},
	{"&Oslash;",  "&#216;",  "Ø", "O - slash"},
	{"&OElig;",   "&#338;",  "Œ", "ligature OE"},
	{"&Scaron;",  "&#352;",  "Š", "S - caron"},
	{"&Ugrave;",  "&#217;",  "Ù", "U - grave"},
	{"&Uacute;",  "&#218;",  "Ú", "U - acute"},
	{"&Ucirc;",   "&#219;",  "Û", "U - circumflex"},
	{"&Uuml;",    "&#220;",  "Ü", "U - diaeresis"},
	{"&Yacute;",  "&#221;",  "Ý", "Y - acute"},
	{"&Yuml;",    "&#376;",  "Ÿ", "Y - diaeresis"},
	{"&THORN;",   "&#222;",  "Þ", "THORN"},
	{"&agrave;",  "&#224;",  "à", "a - grave"},
	{"&aacute;",  "&#225;",  "á", "a - acute"},
	{"&acirc;",   "&#226;",  "â", "a - circumflex"},
	{"&atilde;",  "&#227;",  "ã", "a - tilde"},
	{"&auml;",    "&#228;",  "ä", "a - diaeresis"},
	{"&aring;",   "&#229;",  "å", "a - ring above"},
	{"&aelig;",   "&#230;",  "æ", "ligature ae"},
	{"&ccedil;",  "&#231;",  "ç", "c - cedilla"},
	{"&egrave;",  "&#232;",  "è", "e - grave"},
	{"&eacute;",  "&#233;",  "é", "e - acute"},
	{"&ecirc;",   "&#234;",  "ê", "e - circumflex"},
	{"&euml;",    "&#235;",  "ë", "e - diaeresis"},
	{"&igrave;",  "&#236;",  "ì", "i - grave"},
	{"&iacute;",  "&#237;",  "í", "i - acute"},
	{"&icirc;",   "&#238;",  "î", "i - circumflex"},
	{"&iuml;",    "&#239;",  "ï", "i - diaeresis"},
	{"&eth;",     "&#240;",  "ð", "eth"},
	{"&ntilde;",  "&#241;",  "ñ", "n - tilde"},
	{"&ograve;",  "&#242;",  "ò", "o - grave"},
	{"&oacute;",  "&#243;",  "ó", "o - acute"},
	{"&ocirc;",   "&#244;",  "ô", "o - circumflex"},
	{"&otilde;",  "&#245;",  "õ", "o - tilde"},
	{"&ouml;",    "&#246;",  "ö", "o - diaeresis"},
	{"&oslash;",  "&#248;",  "ø", "o slash"},
	{"&oelig;",   "&#339;",  "œ", "ligature oe"},
	{"&scaron;",  "&#353;",  "š", "s - caron"},
	{"&ugrave;",  "&#249;",  "ù", "u - grave"},
	{"&uacute;",  "&#250;",  "ú", "u - acute"},
	{"&ucirc;",   "&#251;",  "û", "u - circumflex"},
	{"&uuml;",    "&#252;",  "ü", "u - diaeresis"},
	{"&yacute;",  "&#253;",  "ý", "y - acute"},
	{"&thorn;",   "&#254;",  "þ", "thorn"},
	{"&yuml;",    "&#255;",  "ÿ", "y - diaeresis"},
    {"&Alpha;",   "&#913;",  "Α", "Alpha"},
	{"&Beta;",    "&#914;",  "Β", "Beta"},
	{"&Gamma;",   "&#915;",  "Γ", "Gamma"},
	{"&Delta;",   "&#916;",  "Δ", "Delta"},
	{"&Epsilon;", "&#917;",  "Ε", "Epsilon"},
	{"&Zeta;",    "&#918;",  "Ζ", "Zeta"},
	{"&Eta;",     "&#919;",  "Η", "Eta"},
	{"&Theta;",   "&#920;",  "Θ", "Theta"},
	{"&Iota;",    "&#921;",  "Ι", "Iota"},
	{"&Kappa;",   "&#922;",  "Κ", "Kappa"},
	{"&Lambda;",  "&#923;",  "Λ", "Lambda"},
	{"&Mu;",      "&#924;",  "Μ", "Mu"},
	{"&Nu;",      "&#925;",  "Ν", "Nu"},
	{"&Xi;",      "&#926;",  "Ξ", "Xi"},
	{"&Omicron;", "&#927;",  "Ο", "Omicron"},
	{"&Pi;",      "&#928;",  "Π", "Pi"},
	{"&Rho;",     "&#929;",  "Ρ", "Rho"},
	{"&Sigma;",   "&#931;",  "Σ", "Sigma"},
	{"&Tau;",     "&#932;",  "Τ", "Tau"},
	{"&Upsilon;", "&#933;",  "Υ", "Upsilon"},
	{"&Phi;",     "&#934;",  "Φ", "Phi"},
	{"&Chi;",     "&#935;",  "Χ", "Chi"},
	{"&Psi;",     "&#936;",  "Ψ", "Psi"},
	{"&Omega;",   "&#937;",  "Ω", "Omega"},
	{"&alpha;",   "&#945;",  "α", "alpha"},
	{"&beta;",    "&#946;",  "β", "beta"},
	{"&gamma;",   "&#947;",  "γ", "gamma"},
	{"&delta;",   "&#948;",  "δ", "delta"},
	{"&epsilon;", "&#949;",  "ε", "epsilon"},
	{"&zeta;",    "&#950;",  "ζ", "zeta"},
	{"&eta;",     "&#951;",  "η", "eta"},
	{"&theta;",   "&#952;",  "θ", "theta"},
	{"&iota;",    "&#953;",  "ι", "iota"},
	{"&kappa;",   "&#954;",  "κ", "kappa"},
	{"&lambda;",  "&#955;",  "λ", "lambda"},
	{"&mu;",      "&#956;",  "μ", "mu"},
	{"&nu;",      "&#957;",  "ν", "nu"},
	{"&xi;",      "&#958;",  "ξ", "xi"},
	{"&omicron;", "&#959;",  "ο", "omicron"},
	{"&pi;",      "&#960;",  "π", "pi"},
	{"&rho;",     "&#961;",  "ρ", "rho"},
	{"&sigmaf;",  "&#962;",  "ς", "final sigma"},
	{"&sigma;",   "&#963;",  "σ", "sigma"},
	{"&tau;",     "&#964;",  "τ", "tau"},
	{"&upsilon;", "&#965;",  "υ", "upsilon"},
	{"&phi;",     "&#966;",  "φ", "phi"},
	{"&chi;",     "&#967;",  "χ", "chi"},
	{"&psi;",     "&#968;",  "ψ", "psi"},
	{"&omega;",   "&#969;",  "ω", "omega"},
// symbols
	{"&alefsym;", "&#8501;", "ℵ","alef symbol"},
	{"&piv;",     "&#982;",  "ϖ","pi symbol"},
	{"&real;",    "&#8476;", "ℜ","real part symbol"},
	{"&thetasym;","&#977;",  "ϑ","theta symbol"},
	{"&upsih;",   "&#978;",  "ϒ","upsilon - hook symbol"},
	{"&weierp;",  "&#8472;", "℘","Weierstrass p"},
	{"&image;",   "&#8465;", "ℑ","imaginary part"},
// arrows
	{"&larr;",    "&#8592;", "←", "leftwards arrow"},
	{"&uarr;",    "&#8593;", "↑", "upwards arrow"},
	{"&rarr;",    "&#8594;", "→", "rightwards arrow"},
	{"&darr;",    "&#8595;", "↓", "downwards arrow"},
	{"&harr;",    "&#8596;", "↔", "left right arrow"},
	{"&crarr;",   "&#8629;", "↵","carriage return"},
	{"&lArr;",    "&#8656;", "⇐","leftwards double arrow"},
	{"&uArr;",    "&#8657;", "⇑","upwards double arrow"},
	{"&rArr;",    "&#8658;", "⇒","rightwards double arrow"},
	{"&dArr;",    "&#8659;", "⇓","downwards double arrow"},
	{"&hArr;",    "&#8660;", "⇔","left right double arrow"},
	{"&there4;",  "&#8756;", "∴","therefore"},
	{"&sub;",     "&#8834;", "⊂","subset of"},
	{"&sup;",     "&#8835;", "⊃","superset of"},
	{"&nsub;",    "&#8836;", "⊄","not a subset of"},
	{"&sube;",    "&#8838;", "⊆","subset of or equal to"},
	{"&supe;",    "&#8839;", "⊇","superset of or equal to"},
	{"&oplus;",   "&#8853;", "⊕","circled plus"},
	{"&otimes;",  "&#8855;", "⊗","circled times"},
	{"&perp;",    "&#8869;", "⊥","perpendicular"},
	{"&sdot;",    "&#8901;", "⋅","dot operator"},
	{"&lceil;",   "&#8968;", "⌈","left ceiling"},
	{"&rceil;",   "&#8969;", "⌋","right ceiling"},
	{"&lfloor;",  "&#8970;", "⌊","left floor"},
	{"&rfloor;",  "&#8971;", "⌋","right floor"},
	{"&lang;",    "&#9001;", "〈","left-pointing angle bracket"},
	{"&rang;",    "&#9002;", "〉","right-pointing angle bracket"},
	{"&loz;",     "&#9674;", "◊","lozenge"},
	{"&spades;",  "&#9824;", "♠","black spade suit"},
	{"&clubs;",   "&#9827;", "♣", "black club suit"},
	{"&hearts;",  "&#9829;", "♥", "black heart suit"},
	{"&diams;",   "&#9830;", "♦", "black diamond suit"},
	{"&ensp;",    "&#8194;", " ","en space"},
	{"&emsp;",    "&#8195;", " ","em space"},
	{"&thinsp;",  "&#8201;", " ","thin space"},
	{"&zwnj;",    "&#8204;", "‌","zero width non-joiner"},
	{"&zwj;",     "&#8205;", "‍","zero width joiner"},
	{"&lrm;",     "&#8206;", "‎","left-to-right mark"},
	{"&rlm;",     "&#8207;", "‏","right-to-left mark"},
	{"&shy;",     "&#173;",  "­","soft hyphen"}
};

        public string html
        {
            get
            {
                HtmlDocument htmlDoc = this.webBrowser1.Document;
                string XHTML = (string)htmlDoc.InvokeScript("submitForm");
                XHTML = XHTML.Trim();
                //Deletes uneccesary formatting line breaks.
                XHTML = XHTML.Replace("\n", "");
                //Replaces space characters with real space.
                for (int n = 0; n < charmap.GetLength(0); n++)
                {
                    XHTML = XHTML.Replace(charmap[n, 0], charmap[n, 2]);
                    XHTML = XHTML.Replace(charmap[n, 1], charmap[n, 2]);
                }
                return XHTML;
            }
            set
            {
                //for (int n = 0; n < charmap.GetLength(0); n++)
                //{
                    //if (charmap[n, 2] != "")
                    //{
                       // value = value.Replace(charmap[n, 2], charmap[n, 0]);
                       // value = value.Replace(charmap[n, 2], charmap[n, 1]);
                    //}
                //}
                HtmlDocument htmlDoc = this.webBrowser1.Document;
                HtmlElement element = htmlDoc.GetElementById("content"); //htmlDoc.GetElementById("originalText");
                element.SetAttribute("value", value);
                htmlDoc.InvokeScript("setupForm");
            }
        }

        public TinyMCEditor()
        {
            InitializeComponent();
            //this.webBrowser1.Navigate("http://tinymce.moxiecode.com/example_full.php?example=true");
            this.webBrowser1.Navigate(Application.StartupPath + "\\index.html");
            //this.axMozillaBrowser1.Navigate(Application.StartupPath + "\\index.html");
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.ResizeEnd += new System.EventHandler(this.onResizeMe);
        }

        private void buttonOK_Click(object sender, EventArgs e)
        {
            this.Hide();
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            this.Hide();
        }

        private void onResizeMe(object sender, EventArgs e)
        {
        }

        private void Form1_Closing(object sender, FormClosingEventArgs e)
        {
            this.Hide();
            e.Cancel = true;
        }

        //private void buttonOK_Click(object sender, EventArgs e)
        //{
        //    //HtmlDocument htmlDoc = this.webBrowser1.Document;
        //    //htmlDoc.GetElementById("originalText").SetAttribute("value", "New Text");
        //    //htmlDoc.InvokeScript("setupForm");

        //    //HtmlDocument htmlDoc = this.webBrowser1.Document;
        //   ///string html = (string)htmlDoc.InvokeScript("submitForm");
        //}

        ////updates the text in the editor
        //public bool setText(string strText)
        //{
        //    HtmlDocument htmlDoc = this.webBrowser1.Document;
        //    htmlDoc.GetElementById("originalText").SetAttribute("value", "New Text");
        //    htmlDoc.InvokeScript("setupForm");
        //}

        // Updates the URL in TextBoxAddress upon navigation.

    }
}