using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.Runtime.InteropServices;
using System.IO;
using System.Xml;
using System.Xml.XPath;
using System.Xml.Xsl;
using System.Net;

namespace BYOJeopardy_net
{
    public partial class FormEdit : Form
    {
        private byoj_treenode undo_nodes = new byoj_treenode();
        private byoj_treenode redo_nodes = new byoj_treenode();

        private FormExport form_export;
        private FileDialog savedialog_export;
        public FormMain Main = null;
        private bool _saved = true;
        
        private byoj_treenode node_copy;

        public bool saved
        {
            get
            {
                return _saved;
            }
            set
            {
                if (value)
                    this.Text = this.Text.Replace("*", "");
                else
                {
                    if (!this.Text.StartsWith("*"))
                        this.Text = this.Text.Insert(0, "*");
                }
                _saved = value;
            }
        }

        public FormEdit()
        {
            InitializeComponent();
            this.Closing += new CancelEventHandler(this.onClosing);
            FormMain.editor.buttonOK.Click += new System.EventHandler(this.FormEditorbuttonOK_Click);
        }

        private void FormEdit_Load(object sender, EventArgs e)
        {
            has_Undo();
        }

        public void FillTreeView()
        {
            this.treeView1.Nodes.Clear();

            for (int t = 0; t < FormMain.jeopardydata.board.Topics.Length; t++)
            {
                JTopic topic = (JTopic)FormMain.jeopardydata.board.Topics[t];
                byoj_treenode topicNode = new byoj_treenode(topic.name);
                if (topic.name.ToLower() == "daily double")
                    topicNode.node_type = byoj_node_type.Daily_Double;
                else
                    topicNode.node_type = byoj_node_type.Topic;
                for (int v = 0; v < topic.Values.Length; v++)
                {
                    JValue value = topic.Values[v];

                    byoj_treenode valueNode = new byoj_treenode(value.value);
                    valueNode.node_type = byoj_node_type.Value;
                    byoj_treenode questionNode = new byoj_treenode();
                    questionNode.node_type = byoj_node_type.Question;
                    byoj_treenode answerNode = new byoj_treenode();
                    answerNode.node_type = byoj_node_type.Answer;

                    questionNode.Text = value.question;
                    answerNode.Text = value.answer;
                    valueNode.Nodes.Add(questionNode);
                    valueNode.Nodes.Add(answerNode);
                    topicNode.Nodes.Add(valueNode);
                }
                this.treeView1.Nodes.Add(topicNode);
            }
            has_Daily();
        }

        private void Save()
        {
            FormMain.jeopardydata.board.Clear();
            for (int t = 0; t < this.treeView1.Nodes.Count; t++)
            {
                byoj_treenode topicNode = (byoj_treenode)this.treeView1.Nodes[t];
                JTopic topic = new JTopic();
                topic.name = topicNode.Text;
                for (int v = 0; v < topicNode.Nodes.Count; v++)
                {
                    byoj_treenode valueNode = (byoj_treenode)topicNode.Nodes[v];
                    topic.Add(valueNode.Text, valueNode.Nodes[0].Text, valueNode.Nodes[1].Text);
                }
                FormMain.jeopardydata.board.Add(topic);
            }
            FormMain.jeopardydata.SaveJeopardyZipped(FormMain.FileName);
            FormMain.mruMenu.AddFile(FormMain.FileName);
            saved = true;
        }

        private void FormEditorbuttonOK_Click(object sender, EventArgs e)
        {
            FormMain.editor.Hide();
            //this.treeView1.SelectedNode.Text = cleanMediaUrl(FormMain.editor.html);
            //FormMain.editor.SelectedNode.Text = XmlConvert.DecodeName(cleanMediaUrl(XmlConvert.EncodeName(FormMain.editor.html)));
            FormMain.editor.SelectedNode.Text = cleanMediaUrl(FormMain.editor.html);
        }

        private string cleanMediaUrl(string XHTML)
        {
            XmlDataDocument doc = new XmlDataDocument();
            XmlDocumentFragment xElement = doc.CreateDocumentFragment();
            
            //TODO: XHTML does not accept special characters like spaces. &nbsp;           
            xElement.InnerXml = XHTML;
            for (int n = 0; n < xElement.ChildNodes.Count; n++)
            {
                XmlNode src = null;
                if (xElement.ChildNodes[n].Attributes != null)
                {
                    src = xElement.ChildNodes[n].Attributes.GetNamedItem("src");
                    if (src == null && xElement.ChildNodes[n].Attributes.GetNamedItem("name") != null && xElement.ChildNodes[n].Attributes.GetNamedItem("name").Value == "src")
                    {
                        src = xElement.ChildNodes[n].Attributes.GetNamedItem("value");
                    }

                    if (src != null)
                    {
                        string sourceURIFileName = src.Value;
                        if (File.Exists(sourceURIFileName) || Uri.IsWellFormedUriString(sourceURIFileName, UriKind.Absolute))
                        {
                            Uri sourceURI = new Uri(sourceURIFileName);
                            string destFileName = FormMain.tempMediaDir + Path.GetFileName(sourceURI.LocalPath);
                            Uri destURI = new Uri(destFileName);
                            try
                            {
                                File.Copy(sourceURI.LocalPath, destFileName, true);
                            }
                            catch (Exception e)
                            {
                            }
                            //Uri relativeURI = new Uri(FormMain.tempMediaDir);
                            src.Value = destFileName.Replace(FormMain.tempDir, "");
                        }
                    }
                }
                if (xElement.ChildNodes[n].HasChildNodes)
                {
                    xElement.ChildNodes[n].InnerXml = cleanMediaUrl(xElement.ChildNodes[n].InnerXml);
                }
            }
            return xElement.InnerXml;
        }

        
        private string addMediaUrl(string XHTML)
        {
            //changes from a relative path to absolute path
            XmlDataDocument doc = new XmlDataDocument();
            XmlDocumentFragment xElement = doc.CreateDocumentFragment();
            xElement.InnerXml = XHTML;
            for (int n = 0; n < xElement.ChildNodes.Count; n++)
            {
                if (xElement.ChildNodes[n].Attributes != null)
                {
                    XmlNode src = null;
                    if (xElement.ChildNodes[n].Attributes != null)
                    {
                        src = xElement.ChildNodes[n].Attributes.GetNamedItem("src");
                        if (src == null && xElement.ChildNodes[n].Attributes.GetNamedItem("name") != null && xElement.ChildNodes[n].Attributes.GetNamedItem("name").Value == "src")
                        {
                            src = xElement.ChildNodes[n].Attributes.GetNamedItem("value");
                        }
                        if (src != null)
                        {
                            string sourceURIFileName = FormMain.tempDir + src.Value;
                            src.Value = new Uri(sourceURIFileName).LocalPath;
                        }
                    }
                }

                if (xElement.ChildNodes[n].HasChildNodes)
                {
                    xElement.ChildNodes[n].InnerXml = addMediaUrl(xElement.ChildNodes[n].InnerXml);
                }
            }
            return xElement.InnerXml;
        }


        private void addTopic(byoj_treenode topicNode)
        {
            this.treeView1.Nodes.Add(topicNode);
            addUndo((byoj_treenode)topicNode, byoj_node_undo.Paste);
            this.treeView1.SelectedNode = topicNode;
            topicNode.BeginEdit();
        }


        private void addValue(byoj_treenode valueNode, byoj_treenode parentNode)//parentNode or Selected Node
        {
            //Get parent Topic
            this.treeView1.Focus();

            byoj_treenode topicNode;
            if (parentNode == null)
            {
                return;
            }
            else if (parentNode.node_type == byoj_node_type.Daily_Double)
            {
                return;
            }
            else if (parentNode.node_type == byoj_node_type.Topic)
            {
                topicNode = (byoj_treenode)parentNode;
            }
            else if (parentNode.node_type == byoj_node_type.Value)
            {
                topicNode = (byoj_treenode)parentNode.Parent;
            }
            else  //question or answer node
            {
                topicNode = (byoj_treenode)this.treeView1.SelectedNode.Parent.Parent;
            }

            if (topicNode.node_type == byoj_node_type.Daily_Double && topicNode.Nodes.Count > 0)
            {
                return;
            }
            topicNode.Nodes.Add(valueNode);
            addUndo((byoj_treenode)valueNode, byoj_node_undo.Paste);
            this.treeView1.SelectedNode = valueNode;
            valueNode.BeginEdit();
        }

        private void addQuestion_Answer(byoj_treenode questionNode, byoj_treenode parentNode)
        {
            //adds a question or answer
            //tag should equal either "answer" or "question"

            //Get parent Topic
            this.treeView1.Focus();

            byoj_treenode valueNode;
            if (parentNode == null)
            {
                return;
            }
            if (parentNode.node_type == byoj_node_type.Value)
            {
                valueNode = (byoj_treenode)this.treeView1.SelectedNode;
            }
            else if (parentNode.node_type == byoj_node_type.Daily_Double)
            {
                return;
            }
            else if (parentNode.node_type == byoj_node_type.Topic)
            {
                valueNode = (byoj_treenode)this.treeView1.SelectedNode.FirstNode;
            }
            else  //question or answer node
            {
                valueNode = (byoj_treenode)this.treeView1.SelectedNode.Parent;
            }

            for (int t = 0; t < valueNode.Nodes.Count; t++)
            {
                byoj_treenode currentNode = (byoj_treenode)valueNode.Nodes[t];
                if (currentNode.node_type == questionNode.node_type)
                {
                    currentNode.Remove();
                }
            }

            valueNode.Nodes.Insert(0, questionNode);
            addUndo((byoj_treenode)questionNode, byoj_node_undo.Paste);
            this.treeView1.SelectedNode = questionNode;
            questionNode.BeginEdit();
        }


        private bool has_Daily()
        {
            for (int t = 0; t < this.treeView1.Nodes.Count; t++)
            {
                if (((byoj_treenode)this.treeView1.Nodes[t]).node_type == byoj_node_type.Daily_Double && this.treeView1.Nodes[t].Nodes.Count > 0)
                {
                    this.btnAddDaily.Enabled = false;
                    this.addDailyDoubleToolStripMenuItem.Enabled = false;
                    return true;
                }
            }
            this.btnAddDaily.Enabled = true;
            this.addDailyDoubleToolStripMenuItem.Enabled = true;
            return false;
        }

        private void cutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (this.treeView1.SelectedNode == null)
                return;
            byoj_treenode selectedNode = (byoj_treenode)this.treeView1.SelectedNode;
            if (selectedNode.node_type == byoj_node_type.Topic || selectedNode.node_type == byoj_node_type.Daily_Double || selectedNode.node_type == byoj_node_type.Value)
            {
                addUndo(selectedNode, byoj_node_undo.Cut);
                node_copy = (byoj_treenode)selectedNode.CloneAll();
                selectedNode.Remove();
            }
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (this.treeView1.SelectedNode == null)
                return;
            byoj_treenode selectedNode = (byoj_treenode)this.treeView1.SelectedNode;
            node_copy = selectedNode.CloneAll();
        }

        private void pasteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            byoj_treenode paste_node = node_copy.CloneAll();
            if (paste_node.node_type == byoj_node_type.Topic || paste_node.node_type == byoj_node_type.Daily_Double)
            {
                addTopic(paste_node);
            }
            else if (paste_node.node_type == byoj_node_type.Value)
            {
                addValue(paste_node, (byoj_treenode)this.treeView1.SelectedNode);
            }
            else
            {
                addQuestion_Answer(paste_node, (byoj_treenode)this.treeView1.SelectedNode);
            }
            addUndo((byoj_treenode)this.treeView1.SelectedNode, byoj_node_undo.Paste);
            this.treeView1.SelectedNode = paste_node;
        }

        private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (this.treeView1.SelectedNode == null)
                return;
            this.treeView1.SelectedNode.EndEdit(false);

            byoj_treenode selectedNode = (byoj_treenode)this.treeView1.SelectedNode;
            if (selectedNode.node_type == byoj_node_type.Topic || selectedNode.node_type == byoj_node_type.Daily_Double || selectedNode.node_type == byoj_node_type.Value)
            {
                if (selectedNode.node_type == byoj_node_type.Value && ((byoj_treenode)selectedNode.Parent).node_type == byoj_node_type.Daily_Double)
                {
                    //If Daily double value
                    addUndo((byoj_treenode)this.treeView1.SelectedNode.Parent, byoj_node_undo.Cut);
                    this.treeView1.SelectedNode.Parent.Remove();
                }
                else
                {
                    addUndo((byoj_treenode)this.treeView1.SelectedNode, byoj_node_undo.Cut);
                    this.treeView1.SelectedNode.Remove();
                }
            }
            has_Daily();
        }

        private void editLabelToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.treeView1.Focus();
            byoj_treenode parentNode = (byoj_treenode)this.treeView1.SelectedNode;
            if (parentNode.node_type == byoj_node_type.Question | parentNode.node_type == byoj_node_type.Answer)
            {
                FormMain.editor.html = addMediaUrl(this.treeView1.SelectedNode.Text);
                FormMain.editor.SelectedNode = this.treeView1.SelectedNode;
                FormMain.editor.Show();
            }
            else
            {
                if (((byoj_treenode)this.treeView1.SelectedNode).node_type != byoj_node_type.Daily_Double)
                {
                    this.treeView1.SelectedNode.BeginEdit();
                }
            }
            saved = false;
        }

        private void treeView1_AfterLabelEdit(object sender, NodeLabelEditEventArgs e)
        {
            if (!e.CancelEdit && e.Label != null)
            {
                saved = false;
            }

        }

        private void addTopicToolStripMenuItem_Click(object sender, EventArgs e)
        {
            byoj_treenode topicNode = new byoj_treenode("New Topic");
            topicNode.node_type = byoj_node_type.Topic;
            addTopic(topicNode);
        }

        private void addValueToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //Add Value Node
            byoj_treenode valueNode = new byoj_treenode("New Value");
            valueNode.node_type = byoj_node_type.Value;
            byoj_treenode questionNode = new byoj_treenode("New Question");
            questionNode.node_type = byoj_node_type.Question;
            byoj_treenode answerNode = new byoj_treenode("New Answer");
            answerNode.node_type = byoj_node_type.Answer;
            valueNode.Nodes.Add(questionNode);
            valueNode.Nodes.Add(answerNode);
            //topicNode.Expand();
            //valueNode.Expand();
            addValue(valueNode, (byoj_treenode)this.treeView1.SelectedNode);
        }

        private void addDailyDoubleToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (!has_Daily())
            {
                byoj_treenode topicNode = new byoj_treenode("Daily Double");
                topicNode.node_type = byoj_node_type.Daily_Double;
                this.treeView1.Nodes.Add(topicNode);

                //Add Value Node
                byoj_treenode valueNode = new byoj_treenode("New Topic");
                valueNode.node_type = byoj_node_type.Value;
                byoj_treenode questionNode = new byoj_treenode("New Question");
                questionNode.node_type = byoj_node_type.Question;
                byoj_treenode answerNode = new byoj_treenode("New Answer");
                answerNode.node_type = byoj_node_type.Answer;
                valueNode.Nodes.Add(questionNode);
                valueNode.Nodes.Add(answerNode);
                topicNode.Nodes.Add(valueNode);
                topicNode.Expand();
                valueNode.Expand();
                addUndo((byoj_treenode)valueNode, byoj_node_undo.Paste);
                this.treeView1.SelectedNode = valueNode;
                valueNode.BeginEdit();
                this.btnAddDaily.Enabled = false;
            }
        }




        private void has_Undo()
        {
            if (undo_nodes.Nodes.Count > 0)
            {
                this.undoToolStripMenuItem.Enabled = true;
            }
            else
            {
                this.undoToolStripMenuItem.Enabled = false;
            }

            if (redo_nodes.Nodes.Count > 0)
            {
                this.redoToolStripMenuItem.Enabled = true;
            }
            else
            {
                this.redoToolStripMenuItem.Enabled = false;
            }
        }


        private void addUndo(byoj_treenode node, byoj_node_undo undo_type)
        {
            byoj_treenode undo = (byoj_treenode)node.CloneAll();
            undo.node_undo = undo_type;
            undo.origin_parent_node = (byoj_treenode)node.Parent;
            undo.origin_index = node.Index;
            undo.origin_selected_node = (byoj_treenode)this.treeView1.SelectedNode;
            undo_nodes.Nodes.Add(undo);
            redo_nodes.Nodes.Clear();
            has_Undo();
        }

        private void undoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.treeView1.Focus();
            byoj_treenode node = (byoj_treenode)undo_nodes.Nodes[undo_nodes.Nodes.Count - 1];
            byoj_treenode node_clone = node.CloneAll();
            if (node.node_undo == byoj_node_undo.Cut)
            {
                if (node.origin_parent_node != null)
                {
                    node.origin_parent_node.Nodes.Insert(node.origin_index, node_clone);
                }
                else
                {
                    this.treeView1.Nodes.Insert(node.origin_index, node_clone);
                }
                this.treeView1.SelectedNode = node_clone;
            }
            if (node.node_undo == byoj_node_undo.Paste)
            {
                if (node.origin_parent_node != null)
                {
                    node.origin_parent_node.Nodes.RemoveAt(node.origin_index);
                }
                else
                {
                    this.treeView1.Nodes.RemoveAt(node.origin_index);
                }
                if (node.origin_selected_node != null)
                {
                    this.treeView1.SelectedNode = node.origin_selected_node;
                }
            }
            undo_nodes.Nodes[undo_nodes.Nodes.Count - 1].Remove();
            redo_nodes.Nodes.Add(node);
            has_Undo();
        }

        private void redoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.treeView1.Focus();
            byoj_treenode node = (byoj_treenode)redo_nodes.Nodes[redo_nodes.Nodes.Count - 1];
            byoj_treenode node_clone = node.CloneAll();
            if (node.node_undo == byoj_node_undo.Paste)
            {
                if (node.origin_parent_node != null)
                {
                    node.origin_parent_node.Nodes.Insert(node.origin_index, node_clone);
                }
                else
                {
                    this.treeView1.Nodes.Insert(node.origin_index, node_clone);
                }
                this.treeView1.SelectedNode = node_clone;
            }
            if (node.node_undo == byoj_node_undo.Cut)
            {
                if (node.origin_parent_node != null)
                {
                    node.origin_parent_node.Nodes.RemoveAt(node.origin_index);
                }
                else
                {
                    this.treeView1.Nodes.RemoveAt(node.origin_index);
                }
                if (node.origin_selected_node != null)
                {
                    this.treeView1.SelectedNode = node.origin_selected_node;
                }
            }
            redo_nodes.Nodes[redo_nodes.Nodes.Count - 1].Remove();
            undo_nodes.Nodes.Add(node);
            has_Undo();
        }

        private void closeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Hide();
        }

        private void onClosing(object sender, CancelEventArgs e)
        {
            this.Hide();
            // Cancel the Closing event from closing the form.
            e.Cancel = true;
            // Call method to save file...
        }

        private void copyDirectory(string Src, string Dst)
        {
            String[] Files;

            if (Dst[Dst.Length - 1] != Path.DirectorySeparatorChar)
                Dst += Path.DirectorySeparatorChar;
            if (!Directory.Exists(Dst)) Directory.CreateDirectory(Dst);
            Files = Directory.GetFileSystemEntries(Src);
            foreach (string Element in Files)
            {
                // Sub directories
                if (Directory.Exists(Element))
                    copyDirectory(Element, Dst + Path.GetFileName(Element));
                // Files in directory
                else
                    File.Copy(Element, Dst + Path.GetFileName(Element), true);
            }
        }

        private void btnExport_Click(object sender, EventArgs e)
        {
            exportToolStripMenuItem_Click(sender, e);
        }

        private void exportToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (form_export == null)
            {
                form_export = new FormExport();
                string dir = Application.StartupPath + "\\" + "export";
                string[] xsllist = System.IO.Directory.GetFiles(dir, "*.xsl");
                foreach (string fileName in xsllist)
                {
                    XmlDataDocument doc = new XmlDataDocument();
                    doc.Load(fileName);
                    XmlNode docType = doc.FirstChild;
                    XmlNode rootElement = docType.NextSibling;
                    byoj_listviewitem template = new byoj_listviewitem();
                    template.filename = fileName;
                    for (int c = 0; c < rootElement.ChildNodes.Count; c++)
                    {
                        XmlNode node = rootElement.ChildNodes[c];
                        if (node.Name == "xs:template-name")
                        {
                            template.name = node.InnerText;
                            template.Text = node.InnerText;
                        }
                        if (node.Name == "xs:template-description")
                        {
                            template.description = node.InnerText;
                        }
                        if (node.Name == "xs:template-version")
                        {
                            template.version = node.InnerText;
                        }
                        if (node.Name == "xs:template-image")
                        {
                            template.image_string = node.InnerText;
                            if (template.image_string != "")
                            {
                                form_export.imageList1.Images.Add(base64.decodeImage(base64.FixBase64ForImage(template.image_string)));
                                template.ImageIndex = form_export.imageList1.Images.Count - 1;
                            }
                        }
                    }
                    form_export.listView1.Items.Add(template);
                }
            }
            if (FormMain.AnswerasQuestion)
            {
                form_export.rbtnAnswerasQuestion.Checked = true;
            }
            else
            {
                form_export.rbtnAnswerQuestion.Checked = true;
            }
            if (form_export.ShowDialog() == DialogResult.OK)
            {
                if (savedialog_export == null)
                {
                    savedialog_export = new SaveFileDialog();
                    savedialog_export.Filter = "HTML Files(*.html;*.htm)|*.html;*.htm|All Files|*.*";
                }
                if (savedialog_export.ShowDialog() == DialogResult.OK)
                {
                    //open xml file (byojeopardy file)
                    string temp_file = Path.GetTempFileName();
                    FormMain.jeopardydata.LoadFromTreeView(this.treeView1);
                    FormMain.jeopardydata.SaveJeopardy(temp_file);
                    XPathDocument myXPathDoc = new XPathDocument(temp_file);

                    //open xsl file
                    byoj_listviewitem template = (byoj_listviewitem)form_export.listView1.SelectedItems[0];
                    XslCompiledTransform xslt = new XslCompiledTransform();
                    xslt.Load(template.filename);

                    //add arguments to pass to xsl
                    XsltArgumentList argList = new XsltArgumentList();
                    argList.AddParam("xsltFileName", "", template.filename);
                    argList.AddParam("OutputFileName", "", Path.GetFileNameWithoutExtension(savedialog_export.FileName));
                    argList.AddParam("answer_in_form_of_question", "", Convert.ToInt16(form_export.rbtnAnswerasQuestion.Checked));

                    //create stream for output
                    XmlTextWriter writer = new XmlTextWriter(savedialog_export.FileName, null);

                    // Execute the transform.
                    xslt.Transform(myXPathDoc, argList, writer);
                    writer.Close();
                    string outputMediaDir = Path.GetDirectoryName(savedialog_export.FileName) + "\\Pictures";
                    Directory.CreateDirectory(outputMediaDir);
                    this.copyDirectory(FormMain.tempMediaDir, outputMediaDir);
                    File.Delete(temp_file);
                    FormMain.openHTMLinBrowser(savedialog_export.FileName);
                }
            }
        }


        public bool newBYOJeopardy()
        {
            FormTemplate frmTemplate = new FormTemplate();
            if (frmTemplate.ShowDialog() == DialogResult.OK)
            {
                FormMain.FileName = "";
                if (!frmTemplate.rbtnNone.Checked)
                {
                    FormMain.jeopardydata.CreateJeopardy(frmTemplate.Type);
                    this.FillTreeView();
                    frmTemplate.Dispose();
                    return true;
                }
            }
            frmTemplate.Dispose();
            return false;
        }


        public void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            newBYOJeopardy();
        }


        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (FormMain.FileName == "")
            {
                saveAsToolStripMenuItem_Click(sender, e);
            }
            else
            {
                Save();
            }
        }

        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (this.saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                FormMain.FileName = this.saveFileDialog1.FileName;
                Save();
            }
        }

        private void buildBoardToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (!saved)
                MessageBox.Show("This will only show the new board.\nRemember to save the file!",
                "Save me!", MessageBoxButtons.OK, MessageBoxIcon.Information);
            FormFormat Format = new FormFormat();

            if (Format.ShowDialog() == DialogResult.OK)
            {
                FormMain.AnswerasQuestion = Format.AnswerasQuestion.Checked;
                Format.Dispose();
                Main.ClearScreen();
                if (FormMain.jeopardydata.LoadFromTreeView(this.treeView1))
                {
                    Main.buildBoard();
                    Main.Text = "Preview* -- Build Your Own Jeopardy";
                    Main.editToolStripMenuItem.Enabled = true;
                    Main.reloadtoolStripMenuItem.Enabled = false;
                    return;
                }
                else
                {
                    //add Error Dialog

                }
            }
            else
            {
                Format.Dispose();
                return;
            }
            //Board not built
            Main.Text = "Build Your Own Jeopardy";
            Main.editToolStripMenuItem.Enabled = false;
            Main.reloadtoolStripMenuItem.Enabled = false;
            Main.ClearScreen();
        }

        private void sortToolStripMenuItem_Click(object sender, EventArgs e)
        {

            this.treeView1.TreeViewNodeSorter = new CompInv();
            this.treeView1.Sort();
        }

        private void expandOneToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (this.treeView1.SelectedNode == null)
                return;
            this.treeView1.SelectedNode.ExpandAll();
        }

        private void expandAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.treeView1.ExpandAll();
        }

        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            byoj_treenode selectedNode = (byoj_treenode)e.Node;

            this.editLabelToolStripMenuItem.Enabled = true;
            this.editLabelToolStripMenuItem2.Enabled = true;
            this.btnEditLabel.Enabled = true;

            this.copyToolStripMenuItem.Enabled = true;
            this.copyToolStripMenuItem2.Enabled = true;

            this.cutToolStripMenuItem.Enabled = true;
            this.cutToolStripMenuItem2.Enabled = true;

            if (node_copy != null)
            {
                this.pasteToolStripMenuItem.Enabled = true;
                this.pasteToolStripMenuItem2.Enabled = true;
            }
            else
            {
                this.pasteToolStripMenuItem.Enabled = false;
                this.pasteToolStripMenuItem2.Enabled = false;
            }
            
            this.deleteToolStripMenuItem.Enabled = true;
            this.deleteToolStripMenuItem2.Enabled = true;
        }

        private void treeView1_Leave(object sender, EventArgs e)
        {
            if (this.treeView1.SelectedNode == null)
            {
                this.editLabelToolStripMenuItem.Enabled = false;
                this.editLabelToolStripMenuItem2.Enabled = false;
                this.btnEditLabel.Enabled = false;

                this.copyToolStripMenuItem.Enabled = false;
                this.copyToolStripMenuItem2.Enabled = false;

                this.cutToolStripMenuItem.Enabled = false;
                this.cutToolStripMenuItem2.Enabled = false;

                this.pasteToolStripMenuItem.Enabled = false;
                this.pasteToolStripMenuItem2.Enabled = false;

                this.deleteToolStripMenuItem.Enabled = false;
                this.deleteToolStripMenuItem2.Enabled = false;
            }
        }

        private void treeView_ItemDrag(object sender, ItemDragEventArgs e)
        {
           // DoDragDrop(e.Item, DragDropEffects.Move);
        }

        private void treeView1_DragEnter(object sender, DragEventArgs e)
        {
            //e.Effect = DragDropEffects.Move;
        }

        private void treeView1_DragDrop(object sender, DragEventArgs e)
        {
            //byoj_treenode OriginalNode = (byoj_treenode)e.Data.GetData("BYOJeopardy_net.byoj_treenode");
            //OriginalNode.Remove();
            //if (e.Data.GetDataPresent("BYOJeopardy_net.byoj_treenode", false))
            //{
            //    Point pt = ((TreeView)sender).PointToClient(new Point(e.X, e.Y));
            //    byoj_treenode DestinationNode = (byoj_treenode)((TreeView)sender).GetNodeAt(pt);
            //    DestinationNode.Nodes.Add(OriginalNode);
                
            //}
        }

        private void FormEdit_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!saved)
            {
                if (MessageBox.Show("You have not saved the file. \nWould you like to save the file befor closing?",
                    "File Not Saved!", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                {
                    this.saveToolStripMenuItem_Click(sender, null);
                }
            }
            this.Hide();
            e.Cancel = true;
        }
    }

    // Create an IComparer for byoj_treenodes objects. 
    class CompInv : IComparer
    {
        // Implement the IComparable interface. 
        public int Compare(object obj1, object obj2)
        {
            byoj_treenode a, b;
            a = (byoj_treenode)obj1;
            b = (byoj_treenode)obj2;
            if (a.node_type == byoj_node_type.Daily_Double) //moves daily double to bottom
                return 1;
            else if (a.node_type == byoj_node_type.Topic || a.node_type == byoj_node_type.Value)
            {
                int test = -1;
                if (Int32.TryParse(a.Text, out test) && Int32.TryParse(b.Text, out test)) //test for numbers to sort correctly
                    return Convert.ToInt32(a.Text).CompareTo(Convert.ToInt32(b.Text));
                else
                    return a.Text.CompareTo(b.Text);
            }
            else //does not sort question or answer
                return 0;
        }
    }

    public enum byoj_node_undo { Delete, Insert, Cut, Paste };
    public enum byoj_node_type { Topic, Value, Question, Answer, Daily_Double };

    public partial class byoj_treenode : TreeNode
    {
        public byoj_node_type node_type;
        public byoj_node_undo node_undo;
        public byoj_treenode origin_parent_node;
        public byoj_treenode origin_selected_node;
        public int origin_index;

        public byoj_treenode()
        {
        }

        public byoj_treenode(string name)
        {
            this.Text = name;
        }

        public byoj_treenode CloneAll()
        {
            return CloneAll(this);
        }

        private byoj_treenode CloneAll(byoj_treenode parent)
        {
            byoj_treenode myClone = (byoj_treenode)parent.Clone();
            myClone.node_type = parent.node_type;
            myClone.node_undo = parent.node_undo;
            myClone.origin_parent_node = parent.origin_parent_node;
            myClone.origin_selected_node = parent.origin_selected_node;
            myClone.origin_index = parent.origin_index;
            for (int n = 0; n < parent.Nodes.Count; n++)
            {
                myClone.Nodes[n] = CloneAll((byoj_treenode)parent.Nodes[n]);
            }
            return myClone;
        }

    }
}