using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace BYOJeopardy_net
{
    //TODO: add different directory for other media to be imported
    public partial class FormMain : Form
    {
        public static JData jeopardydata = new JData();
        public static bool AnswerasQuestion = true;
        public static string FileName = "";
        public static TinyMCE.TinyMCEditor editor = new TinyMCE.TinyMCEditor();
        public static string tempDir = "";
        public static string tempMediaDir = "";
        public static string tempIndex = "";
        private FormEdit frmEdit;

        public static MruStripMenuInline mruMenu;
        static string mruRegKey = "Software\\WolfeScience\\BYOJeopardy";  // Registry path to keep persistent data
 
        public FormMain()
        {
            InitializeComponent();
            
            setupTemp();
            tableLayoutPanel1.Hide();
            
            mruMenu = new MruStripMenuInline(this.mruToolStripMenuItem, this.firstfiletoolStripMenuItem, new MruStripMenu.ClickedHandler(OnMruFile), mruRegKey + "\\MRU", true);
            for (int n = 0; n <= System.Environment.GetCommandLineArgs().Length - 1; n++)
            {
                Console.WriteLine(System.Environment.GetCommandLineArgs()[n].ToString());
                if (System.IO.Path.GetExtension(System.Environment.GetCommandLineArgs()[n].ToString()) == ".byj")
                {
                    OpenBYOJFile(System.Environment.GetCommandLineArgs()[n].ToString());
                }
            }
        }

        private void createfrmEdit()
        {
            if (frmEdit == null)
            {
                frmEdit = new FormEdit();
                frmEdit.Main = this;
            }
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            createfrmEdit();
            if (frmEdit.newBYOJeopardy())
            {
                frmEdit.Text = "New Board";
                frmEdit.Show();
                frmEdit.BringToFront();
                frmEdit.Activate();
            }
        }

        private void editToolStripMenuItem_Click(object sender, EventArgs e)
        {
            createfrmEdit();
            frmEdit.FillTreeView();
            frmEdit.Show();
            frmEdit.BringToFront();
            frmEdit.Activate();
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (this.openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                OpenBYOJFile(this.openFileDialog1.FileName);
            }
            //Canceled opening
        }

        private void reloadtoolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenBYOJFile(FormMain.FileName);
        }

        private void setupTemp()
        {
            FormMain.tempDir = Path.GetTempPath() + "byj\\";
            FormMain.tempIndex = FormMain.tempDir + "index.html";
            if (Directory.Exists(FormMain.tempDir))
            {
                try
                {
                    File.Delete(FormMain.tempIndex);
                    Directory.Delete(FormMain.tempDir, true);
                }
                catch (Exception e)
                {
                }  
            }
            Directory.CreateDirectory(FormMain.tempDir);
            FileStream tempFile = File.Create(FormMain.tempIndex);
            tempFile.Close();
            FormMain.tempMediaDir = FormMain.tempDir + "Pictures\\";
            Directory.CreateDirectory(FormMain.tempMediaDir);
        }

        public void OpenBYOJFile(string FileNameStr)
        {
            setupTemp();
            FormFormat Format = new FormFormat();
            if (Format.ShowDialog() == DialogResult.OK)
            {
                AnswerasQuestion = Format.AnswerasQuestion.Checked;
                Format.Dispose();
                ClearScreen();
                if (jeopardydata.OpenJeopardyZipped(FileNameStr))
                {
                    buildBoard();
                    createfrmEdit();
                    frmEdit.FillTreeView();
                    this.Text = "Build Your Own Jeopardy -- " + this.openFileDialog1.FileName;
                    this.editToolStripMenuItem.Enabled = true;
                    this.reloadtoolStripMenuItem.Enabled = true;
                    FileName = FileNameStr;
                    mruMenu.AddFile(FileName);
                    return;
                }
            }
            else
            {
                Format.Dispose();
                return;
            }
            closeToolStripMenuItem1_Click(null, null);
        }

        public void ClearScreen()
        {
            tableLayoutPanel1.Hide();
            tableLayoutPanel1.Controls.Clear();
            tableLayoutPanel1.ColumnStyles.Clear();
            tableLayoutPanel1.RowStyles.Clear();
        }

        public void buildBoard()
        {
            //tableLayoutPanel1 = new TableLayoutPanel();
            tableLayoutPanel1.Hide();
            tableLayoutPanel1.BackColor = System.Windows.Forms.Control.DefaultBackColor;  
            tableLayoutPanel1.ColumnCount = jeopardydata.board.Topics.Length;
            tableLayoutPanel1.RowStyles.Add(new RowStyle());

            ///find Daily Double
            int daily_double_count = 0;
            int topics_count = jeopardydata.board.Topics.Length;
            JValue daily_double_value = new JValue();

            for (int c = 0; c < jeopardydata.board.Topics.Length; c++)
            {
                JTopic topic = (JTopic)jeopardydata.board.Topics[c];
                if (topic.name.ToLower() == "daily double")
                {
                    topics_count -= 1;
                    if (topic.Values.Length > 0)
                    {
                        daily_double_value.value = topic.Values[0].value;
                        daily_double_value.question = topic.Values[0].question;
                        daily_double_value.answer = topic.Values[0].answer;
                    }
                }
                else
                {
                    daily_double_count += topic.Values.Length;
                }
            }
            ///set random Daily Double 
            Console.WriteLine("Daily Double Count: " + daily_double_count); 
            Random randy = new Random(0);
            int daily_double = UpperBoundRandoms (daily_double_count - 1);
            Console.WriteLine("Daily Double: " + daily_double);

            ///build board
            daily_double_count = 0;  //reset to find daily double
            for (int c = 0; c < topics_count; c++)
            {
                if (tableLayoutPanel1.ColumnStyles.Count <= c)
                {
                    tableLayoutPanel1.ColumnStyles.Add(new ColumnStyle());
                }
                JTopic topic = (JTopic)jeopardydata.board.Topics[c];
                if (topic.name.ToLower() != "daily double")
                {
                    if (tableLayoutPanel1.RowCount < topic.Values.Length + 1)
                    {
                        tableLayoutPanel1.RowCount = topic.Values.Length + 1;
                    }
                    JLabel lbTopic = new JLabel();
                    lbTopic.Text = topic.name;
                    tableLayoutPanel1.Controls.Add(lbTopic, c, 0);
                    for (int r = 0; r < topic.Values.Length; r++)
                    {
                        if (tableLayoutPanel1.RowStyles.Count <= r + 1)
                        {
                            tableLayoutPanel1.RowStyles.Add(new RowStyle());
                        }
                        JButton btnValue = new JButton();
                        btnValue.Dock = DockStyle.Fill;
                        btnValue.button.Text = topic.Values[r].value;
                        if (daily_double_count == daily_double && daily_double_value.question != "")
                        {
                            btnValue.Topic = daily_double_value.value;
                            btnValue.Question = daily_double_value.question;
                            btnValue.Answer = daily_double_value.answer;
                            btnValue.Daily_Double = true;
                        }
                        else
                        {
                            btnValue.Topic = topic.name;
                                btnValue.Question = (string)topic.Values[r].question;
                                btnValue.Answer = (string)topic.Values[r].answer;
                        }
                        tableLayoutPanel1.Controls.Add(btnValue, c, r + 1);
                        daily_double_count += 1;
                    }
                }
             }
            SizeRows();
            SizeCols();
            this.closeQuestionToolStripMenuItem.Enabled = true;
            tableLayoutPanel1.Show();
        }

        // Generate random numbers with an upper bound specified.
        static int UpperBoundRandoms(int upper)
        {
            // Wait to allow the timer to advance.
            System.Threading.Thread.Sleep(1);

            Random randy = new Random();
            Double randyD = randy.NextDouble();
            int random_number = (int)(randyD * upper);  
            return randy.Next(upper);
        }

        private void SizeRows()
        {
            TableLayoutRowStyleCollection styles = this.tableLayoutPanel1.RowStyles;

            foreach (RowStyle style in styles)
            {
                style.SizeType = SizeType.Percent;
                style.Height = 100 /styles.Count;                  
            }
        }
        private void SizeCols()
        {
            TableLayoutColumnStyleCollection styles = this.tableLayoutPanel1.ColumnStyles;

            foreach (ColumnStyle style in styles)
            {
                style.SizeType = SizeType.Percent;
                style.Width = 100 / styles.Count;  
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormAbout frmAbout = new FormAbout();
            frmAbout.ShowDialog(); 
        }

        private void FormMain_Load(object sender, EventArgs e)
        {

        }

        private void updatesToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            FormUpdates dialog = new FormUpdates();
            dialog.ShowDialog(); ;
        }

        private void forumToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            FormForum dialog = new FormForum();
            dialog.ShowDialog(); ;
        }

        public static void openHTMLinBrowser(string site)
        {
            System.Diagnostics.Process p = new System.Diagnostics.Process();
            System.Diagnostics.ProcessStartInfo pi =
            new System.Diagnostics.ProcessStartInfo();
            pi.FileName = site;
            pi.UseShellExecute = true;
            p.StartInfo = pi;
            p.Start();
        }

        private void closeToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            //File not opened
            this.Text = "Build Your Own Jeopardy";
            FileName = "";
            this.editToolStripMenuItem.Enabled = false;
            this.reloadtoolStripMenuItem.Enabled = false;
            ClearScreen();
            if (this.frmEdit.Visible)
                this.frmEdit.Hide();
            this.closeQuestionToolStripMenuItem.Enabled = false;
        }

        private void importToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (this.importFileDialog.ShowDialog() == DialogResult.OK)
            {
                FileName = "";
                if (Path.GetExtension(this.importFileDialog.FileName) == ".ppt")
                {
                    this.importFileDialog.FileName = OOfficeLibrary.Convert.importMicrosoftOfficeFile(this.importFileDialog.FileName);
                    jeopardydata.ImportOpenDocumentPresentation(this.importFileDialog.FileName);
                    File.Delete(this.importFileDialog.FileName);
                }
                else
                {
                    jeopardydata.ImportOpenDocumentPresentation(this.importFileDialog.FileName);
                }
                createfrmEdit();
                frmEdit.FillTreeView();
                frmEdit.Show();
                frmEdit.BringToFront();
                frmEdit.Activate();
            }
        }

        //protected override void OnMouseClick(System.Windows.Forms.MouseEventArgs e)
        //{
        //    System.Console.WriteLine("Musa Clicked Main");
        //}

        //private void tableLayoutPanel1_Click(object sender, EventArgs e)
        //{
        //    object tableobj = this.tableLayoutPanel1.GetChildAtPoint(MousePosition);
        //    if (tableobj.GetType() == Type.GetType("JButton"))
        //    {
        //        Console.WriteLine("Goh");
        //    }
        //}
        private void OnMruFile(int number, String filename)
        {
            if (File.Exists(filename))
            {
                this.OpenBYOJFile(filename);
            }
            else
            {
                MessageBox.Show("The file '" + filename + "'cannot be opened and will be removed from the Recent list(s)"
                    , "BYOJeopardy"
                    , MessageBoxButtons.OK
                    , MessageBoxIcon.Error);
                mruMenu.RemoveFile(number);
            }
        }

        private void FormMain_Closing(object sender, FormClosingEventArgs e)
        {
            mruMenu.SaveToRegistry();
        }

    }

}
//works on command line: C:\>"C:\Program Files\OpenOffice.org 2.0\program\soffice" -invisible "C:\convertooo.odt" "macro://convertooo/Standard.Convert.SaveAsOOO(c:\ppt\germany.ppt)"
