using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.IO;
using System.Data;
using System.Xml;
using System.Windows.Forms;
using System.Drawing;

namespace BYOJeopardy_net
{
    public class JData
    {
        public JBoard board = new JBoard();
        public JData()
        {
        }

        public bool LoadFromTreeView(TreeView treeview)
        {
            FormProgress progress = new FormProgress();
            progress.Text = "Loading Board";
            progress.Show();
            try
            {
                board.Clear();
                //Load File

                for (int n = 0; n < treeview.Nodes.Count; n++)
                {
                    //ADD TOPICS
                    progress.progressBar1.Value = Convert.ToInt32(((float)n / (float)treeview.Nodes.Count) * 100);

                    byoj_treenode topicNode = (byoj_treenode)treeview.Nodes[n];

                    JTopic newtopic = new JTopic();
                    if (topicNode.node_type == byoj_node_type.Topic || topicNode.node_type == byoj_node_type.Daily_Double)
                    {
                        newtopic.name = topicNode.Text;
                        Console.WriteLine("topic: " + topicNode.Text);
                    }
                    for (int c = 0; c < topicNode.Nodes.Count; c++)
                    {
                        JValue newValue = new JValue();
                        progress.progressBar1.Value = Convert.ToInt32(((float)n / (float)treeview.Nodes.Count) * 100 + ((float)(c + 1) / (float)topicNode.Nodes.Count) * ((float)1 / (float)treeview.Nodes.Count) * 100);
                        byoj_treenode valueNode = (byoj_treenode)topicNode.Nodes[c];
                        Console.WriteLine("amount: " + valueNode.Text);
                        newValue.value = valueNode.Text;
                        for (int r = 0; r < valueNode.Nodes.Count; r++)
                        {
                            byoj_node_type node_type = ((byoj_treenode)valueNode.Nodes[r]).node_type;
                            switch (node_type)
                            {
                                case byoj_node_type.Question:
                                    Console.WriteLine("question: " + valueNode.Nodes[r].Text);
                                    newValue.question = valueNode.Nodes[r].Text;
                                    break;
                                case byoj_node_type.Answer:
                                    Console.WriteLine("answer: " + valueNode.Nodes[r].Text);
                                    newValue.answer = valueNode.Nodes[r].Text;
                                    break;
                                default:
                                    break;
                            }
                        }
                        newtopic.Add(newValue);
                    }
                    board.Add(newtopic);
                }

                progress.progressBar1.Value = 100;
            }

            catch (Exception e)
            {
                Console.WriteLine("Error: loading file..." + e.Message);
                MessageBox.Show("Error: loading BYOJeopardy from treeview", "Error Loading File", MessageBoxButtons.OK, MessageBoxIcon.Error);
                progress.Dispose();
                return false;
            }
            progress.Dispose();
            return true;
        }

        public bool OpenFile_as_not_XML(string FileName)
        {
            try
            {
                board.Clear();
                //Open File
                StreamReader JeopardyFile = new StreamReader(FileName);
                string Line = JeopardyFile.ReadLine();
                if (!(Line == null))
                {
                    do
                    {
                        //ADD TOPICS
                        JTopic newTopic = new JTopic();
                        newTopic.name = Line;
                        //ADD Values;Questions;Answers
                        Line = JeopardyFile.ReadLine();
                        if (!(Line == null))
                        {
                            do
                            {
                                JValue newValue = new JValue();
                                string[] LineArray = Line.Split(Convert.ToChar(";"));
                                if (LineArray.Length > 1)
                                {
                                    newValue.value = LineArray[0].ToString();
                                    newValue.question = LineArray[1].ToString();
                                    newValue.answer = LineArray[2].ToString();
                                }
                                else
                                {
                                    goto EndTopic;
                                }
                                newTopic.Add(newValue);
                                Line = JeopardyFile.ReadLine();
                            } while (!(Line == null));
                        }
                    EndTopic: ;
                        board.Add(newTopic);
                    } while (!(Line == null));
                }
                JeopardyFile.Close();
            }
            catch (Exception e)
            {
                Console.WriteLine("Error: opening file..." + e.Message);
                MessageBox.Show("Error: opening BYOJeopardy text file", "Error Opening File", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
            return true;
        }


        public bool OpenFile(string FileName)
        {
            FormProgress progress = new FormProgress();
            progress.Text = "Opening File";
            progress.Show();
            try
            {
                board.Clear();
                //Open File
                XmlDataDocument doc = new XmlDataDocument();
                doc.Load(FileName);
                XmlNode docType = doc.FirstChild;
                XmlNode rootElement = docType.NextSibling;
                decimal version = (decimal)Convert.ToDecimal(rootElement.Attributes.GetNamedItem("version").InnerText);
                Console.WriteLine(version);
                if (version >= 1)
                {
                    XmlNodeList topicElements = rootElement.ChildNodes;

                    for (int n = 0; n < topicElements.Count; n++)
                    {
                        //ADD TOPICS
                        progress.progressBar1.Value = Convert.ToInt32(((float)n / (float)topicElements.Count) * 100);

                        XmlNode topicElement = topicElements[n];
                        XmlNodeList topicChildren = topicElement.ChildNodes;

                        JTopic newTopic = new JTopic();
                        for (int c = 0; c < topicChildren.Count; c++)
                        {
                            progress.progressBar1.Value = Convert.ToInt32(((float)n / (float)topicElements.Count) * 100 + ((float)(c + 1) / (float)topicChildren.Count) * ((float)1 / (float)topicElements.Count) * 100);
                            XmlNode topicChild = topicChildren[c];

                            if (topicChild.Name == "topic_name")
                            {
                                Console.WriteLine("topic_name: " + topicChild.InnerText);
                                newTopic.name = topicChild.InnerText;
                            }
                            else //value
                            {
                                XmlNodeList valueElements = topicChild.ChildNodes;
                                JValue newValue = new JValue();
                                for (int r = 0; r < valueElements.Count; r++)
                                {
                                    XmlNode valueChild = valueElements[r];
                                    switch (valueChild.Name)
                                    {
                                        case "amount":
                                            Console.WriteLine("amount: " + valueChild.InnerText);
                                            newValue.value = valueChild.InnerText;
                                            break;
                                        case "question":
                                            Console.WriteLine("question: " + valueChild.InnerText);
                                            newValue.question = valueChild.InnerText;
                                            break;
                                        case "answer":
                                            Console.WriteLine("answer: " + valueChild.InnerText);
                                            newValue.answer = valueChild.InnerText;
                                            break;
                                        default:
                                            break;
                                    }
                                }
                                newTopic.Add(newValue);
                            }
                        }
                        board.Add(newTopic);
                    }
                
                    progress.progressBar1.Value = 100;
                }
            }
            catch (System.Xml.XmlException eXml)
            {
                Console.WriteLine("Error: opening file...Xml Exception: " + eXml.Message);
                if (eXml.Message.IndexOf("Data at the root level is invalid") > -1)
                {
                    //try to open the old file format
                    progress.Dispose();
                    return OpenFile_as_not_XML(FileName);
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("Error: opening file..." + e.Message);
                MessageBox.Show("Error: opening BYOJeopardy xml file", "Error Opening File", MessageBoxButtons.OK, MessageBoxIcon.Error);
                progress.Dispose();
                return false;
            }
            progress.Dispose();
            return true;
        }

        public bool OpenJeopardyZipped(string FileName)
        {
            FormProgress progress = new FormProgress();
            progress.Text = "UnZipping Files";
            progress.Show();
            progress.progressBar1.Value = 25;
            string FileNameXML;
            try
            {
                //Create Zip Archive
                ICSharpCode.SharpZipLib.Zip.FastZip zipArchive = new ICSharpCode.SharpZipLib.Zip.FastZip();
                zipArchive.ExtractZip(FileName, FormMain.tempDir, "");
                progress.progressBar1.Value = 75;
                FileNameXML = FormMain.tempDir + "content.xml";
            }
            catch (ICSharpCode.SharpZipLib.Zip.ZipException eZip)
            {
                if (eZip.Message == "Wrong Local header signature: 0x6D783F3C")
                {
                    FileNameXML = FileName;
                }
                else
                {
                    MessageBox.Show("Error: Unzipping BYOJeopardy file, Unknown Zip Error: " + eZip.Message, "Error Unzipping File", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    progress.Dispose();
                    return false;
                }
            }
            catch (Exception e)
            {
                MessageBox.Show("Error: Unzipping BYOJeopardy file, General Error: " + e.Message, "Error Unzipping File", MessageBoxButtons.OK, MessageBoxIcon.Error);
                progress.Dispose();
                return false;
            }
            progress.progressBar1.Value = 100;
            //this directory should be deleted and then created whenever there is a new file or a file is opened
            progress.Dispose();
            return OpenFile(FileNameXML);
        }

        public bool ImportOpenDocumentPresentation(string FileName)
        {
            FormProgress progress = new FormProgress();
            progress.Text = "UnZipping--OpenDocument Presentation Files";
            progress.Show();
            progress.progressBar1.Value = 25;
            board.Clear();

            string FileNameXML;
            //TODO: Delete directory when done;
            string OpenDocumentTempDir = Path.GetTempPath() + Path.GetFileNameWithoutExtension(FileName);
            try
            {
                //Create Zip Archive
                ICSharpCode.SharpZipLib.Zip.FastZip zipArchive = new ICSharpCode.SharpZipLib.Zip.FastZip();
                zipArchive.ExtractZip(FileName, OpenDocumentTempDir, "");
                progress.progressBar1.Value = 50;
            }
            catch (ICSharpCode.SharpZipLib.Zip.ZipException eZip)
            {
                Console.WriteLine("Error: unzipping files, Unknown Zip Error: " + eZip.Message);
                return false;
            }
            catch (Exception e)
            {
                Console.WriteLine("Error: unzipping files, General Error: " + e.Message);
                    MessageBox.Show("Error: Unzipping ODP file, General Error: " + e.Message + "\n Check to make sure the file is not open in another program", "Error Unzipping ODP File", MessageBoxButtons.OK, MessageBoxIcon.Error);
                progress.Dispose();
                return false;
            }

            //copy images to directory and convert images 
            progress.Text = "UnZipping--Converting wmf files to png";
            progress.progressBar1.Value = 60;
            if (Directory.Exists(OpenDocumentTempDir + "\\Pictures"))
            {
                string[] filePictures = Directory.GetFiles(OpenDocumentTempDir + "\\Pictures", "*.wmf");
                foreach (string imageFileName in filePictures)
                {
                    Bitmap image = new Bitmap(imageFileName);
                    image.Save(FormMain.tempMediaDir + Path.GetFileNameWithoutExtension(imageFileName) + ".png", System.Drawing.Imaging.ImageFormat.Png);
                    File.Delete(imageFileName);
                }
                progress.Text = "UnZipping--Copying jpg, png and gif files";
                progress.progressBar1.Value = 70;
                filePictures = Directory.GetFiles(OpenDocumentTempDir + "\\Pictures");
                foreach (string imageFileName in filePictures)
                {
                    if (File.Exists(FormMain.tempMediaDir + Path.GetFileName(imageFileName)))
                    {
                        File.Copy(imageFileName, FormMain.tempMediaDir + Path.GetFileName(imageFileName));
                    }
                }
            }
            progress.Text = "UnZipping--Importing slides";
            progress.progressBar1.Value = 80;

            XmlDataDocument doc = new XmlDataDocument();
            doc.Load(OpenDocumentTempDir + "\\content.xml");
            XmlNodeList pages = doc.GetElementsByTagName("draw:page");
            ArrayList Values = new ArrayList();
            for(int n = 0; n < pages.Count; n++)
            {
                XmlElement page = (XmlElement)pages[n];
                if (n == 0)
                {
                    // XmlElement pageElements = (XmlElement)page;

                    XmlNodeList buttons = page.GetElementsByTagName("draw:custom-shape");
                    foreach (XmlElement button in buttons)
                    {
                        XmlNodeList events = button.GetElementsByTagName("presentation:event-listener");
                        if (events.Count == 0)
                        {
                            //this is a topic
                            JTopic newTopic = new JTopic();
                            newTopic.svgX = Convert.ToSingle(button.Attributes["svg:x"].Value.Replace("cm", ""));
                            newTopic.svgY = Convert.ToSingle(button.Attributes["svg:y"].Value.Replace("cm", ""));
                            newTopic.name = button.InnerText;
                            board.Add(newTopic);
                        }
                        else
                        {
                            //this is a value
                            JValue newValue = new JValue();
                            Values.Add(newValue);
                            newValue.svgX = Convert.ToSingle(button.Attributes["svg:x"].Value.Replace("cm", ""));
                            newValue.svgY = Convert.ToSingle(button.Attributes["svg:y"].Value.Replace("cm", ""));
                            newValue.value = button.InnerText;
                            foreach (XmlElement findpage in events)
                            {
                                string link = findpage.Attributes["xlink:href"].Value.Replace("#", "");
                                if (link != null)
                                {
                                    newValue.page = link;
                                }
                            }
                        }
                    }
                }
                else
                {
                    foreach (JValue value in Values)
                    {
                        //This has errors sometimes
                        //maybe I should use direct positions for page# or the link in the text on page #1
                        if (page.Attributes["draw:name"].Value == value.page) 
                        {
                            //Add answer to value
                            value.answer = page.InnerText;
                            XmlNodeList images = page.GetElementsByTagName("draw:image");
                            foreach (XmlElement image in images)
                            {
                                if (image.Attributes["xlink:href"] != null)
                                {
                                    string imageLocation = image.Attributes["xlink:href"].Value.Replace(".wmf", ".png");
                                    value.answer += "<p><img src='" + imageLocation + "' alt='" + imageLocation + "' title='" + imageLocation + "' /></p>";
                                }
                              }
                              
                            //Add question to value
                            //goto next page
                              n += 1;
                              page = (XmlElement)pages[n];
                            value.question = page.InnerText;
                            images = page.GetElementsByTagName("draw:image");
                            foreach (XmlElement image in images)
                            {
                                if (image.Attributes["xlink:href"].Value != null)
                                {
                                    string imageLocation = image.Attributes["xlink:href"].Value.Replace(".wmf", ".png");
                                    value.question += "<p><img src='" + imageLocation + "' alt='" + imageLocation + "' title='" + imageLocation + "' /></p>";
                                }
                            }
                        }
                    }
                }
            }
            //sort values based on svgY
            Values.Sort(new CompSvgY());
            //Add Values to the correct topics
            foreach (JValue value in Values)
            {
                foreach (JTopic topic in board.Topics)
                {
                    if (topic.svgX == value.svgX)
                    {
                        topic.Add(value);
                    }
                }
            }


            progress.progressBar1.Value = 100;
            //this directory should be deleted and then created whenever there is a new file or a file is opened
            //if (OpenFile(FileNameXML) == false)
            //{
            //TODO: show error here;
            //    return false;
            //}
            progress.Dispose();
            return true;
        }


        public bool SaveJeopardy_not_as_XML(string FileName)
        {
            try
            {
                StreamWriter JeopardyFile = new StreamWriter(FileName);
                for (int t = 0; t < board.Topics.Length; t++)
                {
                    JTopic topic = (JTopic)board.Topics[t];
                    JeopardyFile.WriteLine(topic.name);
                    for (int v = 0; v < topic.Values.Length; v++)
                    {
                        JValue value = topic.Values[v];
                        JeopardyFile.WriteLine(value.value + ";" + value.question + ";" + value.answer);
                    }
                }
                JeopardyFile.Close();
            }
            catch (Exception e)
            {
                Console.WriteLine("Error: saving file..." + e.Message);
                MessageBox.Show("Error: saving BYOJeopardy xml file", "Error Saving File", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
            return true;
        }

        public bool SaveJeopardy(string FileName)
        {
            FormProgress progress = new FormProgress();
            progress.Text = "Saving File";
            progress.Show();
            try
            {
                //Create an XmlDataDocument.
                XmlDataDocument doc = new XmlDataDocument();
                XmlProcessingInstruction instruction = doc.CreateProcessingInstruction("xml",
                        " version='1.0' encoding='UTF-8'");
                doc.AppendChild(instruction);

                XmlNode rootElement = doc.CreateNode(XmlNodeType.Element, "byojeopardy", "");

                XmlAttribute version = doc.CreateAttribute("version");
                XmlText versiontext = doc.CreateTextNode("1.0");
                version.AppendChild(versiontext);
                rootElement.Attributes.Append(version);

                doc.AppendChild(rootElement);

                for (int t = 0; t < board.Topics.Length; t++)
                {
                    progress.progressBar1.Value = Convert.ToInt32(((float)t / (float)board.Topics.Length) * 100);
                    JTopic topic = (JTopic)board.Topics[t];
                    XmlNode topicElement = doc.CreateNode(XmlNodeType.Element, "topic", "");

                    XmlNode topicnameElement = doc.CreateNode(XmlNodeType.Element, "topic_name", "");
                    XmlText topicnametext = doc.CreateTextNode(topic.name);
                    topicnameElement.AppendChild(topicnametext);
                    topicElement.AppendChild(topicnameElement);


                    for (int v = 0; v < topic.Values.Length; v++)
                    {
                        progress.progressBar1.Value = Convert.ToInt32(((float)t / (float)board.Topics.Length) * 100 + ((float)(v + 1) / (float)topic.Values.Length) * ((float)1 / (float)board.Topics.Length) * 100);
                        JValue value = topic.Values[v];

                        //    XmlText topic1 = BYOJeopardyDoc.CreateTextNode("topic");
                        XmlNode valueElement = doc.CreateNode(XmlNodeType.Element, "value", "");

                        XmlNode valueAmountElement = doc.CreateNode(XmlNodeType.Element, "amount", "");
                        XmlText valuetext = doc.CreateTextNode(value.value);
                        valueAmountElement.AppendChild(valuetext);
                        valueElement.AppendChild(valueAmountElement);

                        XmlNode questionElement = doc.CreateNode(XmlNodeType.Element, "question", "");
                        XmlText questiontext = doc.CreateTextNode(value.question);
                        questionElement.AppendChild(questiontext);
                        valueElement.AppendChild(questionElement);

                        XmlNode answerElement = doc.CreateNode(XmlNodeType.Element, "answer", "");
                        XmlText answertext = doc.CreateTextNode(value.answer);
                        answerElement.AppendChild(answertext);
                        valueElement.AppendChild(answerElement);

                        topicElement.AppendChild(valueElement);
                    }
                    rootElement.AppendChild(topicElement);
                }
                progress.progressBar1.Value = 100;
                doc.Save(FileName);
            }
            catch (Exception e)
            {
                Console.WriteLine("Error: saving file..." + e.Message);
                MessageBox.Show("Error: saving BYOJeopardy xml file", "Error Saving File", MessageBoxButtons.OK, MessageBoxIcon.Error);
                progress.Dispose();
                return false;
            }
            progress.Dispose();
            return true;
        }

        public bool SaveJeopardyZipped(string FileName)
        {
            //this directory should be deleted and then created whenever there is a new file or a file is opened
            string FileNameXML = FormMain.tempDir + "content.xml";
            if (SaveJeopardy(FileNameXML) == false)
                return false;

            FormProgress progress = new FormProgress();
            progress.Text = "Zipping Files";
            progress.Show();
            try
            {
                //Create Zip Archive
                ICSharpCode.SharpZipLib.Zip.FastZip zipArchive = new ICSharpCode.SharpZipLib.Zip.FastZip();
                zipArchive.CreateZip(FileName, FormMain.tempDir, true, "");
            }
            catch (ICSharpCode.SharpZipLib.Zip.ZipException ex)
            {
                Console.WriteLine("Error: saving zipped files..." + ex.Message);
                MessageBox.Show("Error: saving BYOJeopardy zipped file", "Error Saving File", MessageBoxButtons.OK, MessageBoxIcon.Error);
                progress.Dispose();
                return false;
            }
            progress.Dispose();
            return true;
        }

        public void CreateJeopardy(int Type)
        {
            int cashValue=0;
            switch (Type)
            {
                case 0://Type==0 original board
                    {
                        cashValue = 100;
                        break;
                    }
                case 1://Type==1 double jeopardy
                    {
                        cashValue = 200;
                        break;
                    }
                case 2://Type==2 5 point system
                    {
                        cashValue = 1;
                        break;
                    }
                case 3://Type==3 10 point system
                    {
                        cashValue = 2;
                        break;
                    }
            }

            board.Clear();
            for (int t = 1; t < 6; t++)
            {
                JTopic topic = new JTopic();
                topic.name = "Topic" + t.ToString();
                for (int v = 1; v < 6; v++)
                {
                    JValue value = new JValue();
                    string valueString = (v * cashValue).ToString();
                    if (Type > 1)
                    {
                        valueString += " pt";
                    }
                    value.value = valueString;
                    value.question = "Question " + valueString;
                    value.answer = "Answer " + valueString;
                    topic.Add(value);
                }
                board.Add(topic);
            }
            JTopic daily_double = new JTopic();
            daily_double.name = "Daily Double";
            JValue daily_double_value = new JValue();
            daily_double_value.value = "Daily Topic";
            daily_double_value.question = "Daily Question";
            daily_double_value.answer = "Daily Answer";
            daily_double.Add(daily_double_value);
            board.Add(daily_double);
        }
    }

    public class JBoard
    {
        public JTopic[] Topics = new JTopic[0];
        public void Add(JTopic topic)
        {
            JTopic[] newTopics = new JTopic[Topics.Length + 1];
            Topics.CopyTo(newTopics, 0);
            newTopics[Topics.Length] = topic;
            Topics = newTopics;
        }
        public void Clear()
        {
            Topics = new JTopic[0];
        }
    }
    public class JTopic
    {
        public string name = "";

        public JValue[] Values = new JValue[0];
        public void Add(string value, string question, string answer)
        {
            JValue[] newValues = new JValue[Values.Length + 1];
            Values.CopyTo(newValues, 0);
            JValue newValue = new JValue();
            newValue.answer = answer;
            newValue.question = question;
            newValue.value = value;
            newValues[Values.Length] = newValue;
            Values = newValues;
        }
        public void Add(JValue value)
        {
            JValue[] newValues = new JValue[Values.Length + 1];
            Values.CopyTo(newValues, 0);
            newValues[Values.Length] = value;
            Values = newValues;
        }
        public void Clear()
        {
            Values = new JValue[0];
        }
        //for OpenDocumentPresentation Importing
        public Single svgX = 0;
        public Single svgY = 0;
    }
    public class JValue: Object
    {
        public string value = "";
        public string question = "";
        public string answer = "";
        //for OpenDocumentPresentation Importing
        public string page; 
        public Single svgX = 0;
        public Single svgY = 0;
    }

    // Create an IComparer for OpenDocumentPresentation Importing 
    class CompSvgY : IComparer
    {
        //sort values based on svgY
        public int Compare(object obj1, object obj2)
        {
            JValue a, b;
            a = (JValue)obj1;
            b = (JValue)obj2;
            if (a.svgY != b.svgY)
                return a.svgY.CompareTo(b.svgY);
            else
                return a.svgX.CompareTo(b.svgX); ;
        }
    }
}
